==> Cosmos7.61.tar.gz: Cosmos7.61/Import/PHITS/cphitsOut.f <==
      subroutine cphitsOut(n, a)
c       **** note  param00.inc by phits dose not accept implicit none
c            a-h,o-z: double.  i-m; integer
c            New variables here must be explicitly declared
c            without exception 
c      implicit none
#include "Zcode.h"
#include "Zptcl.h"
      include "JQMD/param00.inc"
      integer,intent(out)::n  ! total number of ptcls 
      record /ptcl/ a(*)       !   a(n). each ptcl property
      integer code, subcode, charge

      integer nclst, iclust
*        nclst   : total number of out going particles and nuclei      *
      common /clustf/ nclst, iclust(nnn)
      integer jclust
      real(8):: qclust
      common /clustg/ jclust(0:7,nnn),  qclust(0:12,nnn)
      integer,save:: nstrange= 0
      integer i, j
*      should be  j=  iclust(nnn)
*                j = 0, nucleus                                        *
*                  = 1, proton                                         *
*                  = 2, neutron                                        *
*                  = 3, pion                                           *
*                  = 4, photon                                         *
*                  = 5, kaon                                           *
*                  = 6, muon                                           *
*                  = 7, others                                         *
*                                                                      *

*          jclust(j,:)                                                 *
*                j = 0, angular momentum                               *
*                  = 1, proton number                                  *
*                  = 2, neutron number                                 *
*                  = 3, ip, see below                                  *
*                  = 4,                                                *
*                  = 5, charge                                         *
*                  = 6, baryon number                                  *
*                  = 7, kf code                                        *
*                                                                      *
*        qclust(i,nclst)                                               *
*                                                                      *
*                i = 0, impact parameter                               *
*                  = 1, px (GeV/c)                                     *
*                  = 2, py (GeV/c)                                     *
*                  = 3, pz (GeV/c)                                     *
*                  = 4, etot = sqrt( p**2 + rm**2 ) (GeV)              *
*                  = 5, rest mass (GeV)                                *
*                  = 6, excitation energy (MeV)                        *
*                  = 7, kinetic energy (MeV)                           *
*                  = 8, weight change                                  *
*                  = 9, delay time                                     *
*                  = 10, x-displace                                    *
*                  = 11, y-displace                                    *
*                  = 12, z-displace                                    *
*                                                                      *
*        numpat(i) : total number of out going particles or nuclei     *
*                                                                      *
*                i =  0, nuclei                                        *
*                  =  1, proton                                        *
*                  =  2, neutron                                       *
*                  =  3, pi+                                           *
*                  =  4, pi0                                           *
*                  =  5, pi-                                           *
*                  =  6, mu+                                           *
*                  =  7, mu-                                           *
*                  =  8, K+                                            *
*                  =  9, K0                                            *
*                  = 10, K-                                            *
*                                                                      *
*                  = 11, other particles                               *
*                                                                      *
*                  = 12, electron                                      *
*                  = 13, positron                                      *
*                  = 14, photon                                        *
*                                                                      *
*                  = 15, deuteron                                      *
*                  = 16, triton                                        *
*                  = 17, 3He                                           *
*                  = 18, Alpha                                         *
*                  = 19, residual nucleus                              *
*                                                                      *
*                                                                      *
************************************************************************
      integer kfcode
c/////////
c      write(0,*) 'cphitsOut:  nclst=', nclst
c      write(0,*) '  iclust =',  iclust(1:nclst)
c//////////
      n = 0
      do i=1, nclst
c//////////
c         write(0,*) 'i=', i, ' jclust=',jclust(0:7,i)
c///////////
         if( iclust(i) == 0 ) then
            ! nucleus  but sometimes  p/n comes out  or even
            !  strange ptcl  ptroton =2  nueutorn = -1 !
            charge = jclust(5,i)
            subcode = jclust(6,i)
c///////////
c            write(0,*) ' jclust,6,5=',subcode, charge
c/////////
            if( subcode == 1 ) then
               ! should be n or p; if |charge|>1, will be detected later
               code = knuc
               subcode = -1
              ! later charge is checked and some times strange
            else 
               code = kgnuc
            endif
         else
            kfcode=jclust(7,i)
c/////////////
c            write(0,*) 'i=',i, "  kfcode =",kfcode, 
c     *           ' iclust=', iclust(i)
c/////////////
            call ckf2cos(kfcode, code, subcode, charge)
         endif
         if( code /= krare ) then
            call cmkptc(code, subcode, charge, a(i))
            if( abs(charge) > 1 .and. code == 6) then
            !  seems to appear for pi- proj.
            !       chg=2. i is normally
            !      last one. chg=3,4 is also seen.
!               if( charge == 2 .and.
                 if( abs( qclust(5,i)- 0.9385d0) < 1.d-3 ) then
                          !  mass
                    if( abs( qclust(4,i)-qclust(5,i)-qclust(7,i)/1.d3)
                        !          T.E  - mass      - K.E (MeV) 
     *                  < 1.d-3 ) then
            !   charge >= 1 code =6 seems proton (since mass is p)
            !   and energy is mass + K.E so we assign it to p

                    charge = 1
                    call cmkptc(code, subcode, charge, a(i))
                    goto 333
                 endif
              endif
              nstrange = nstrange + 1
              if(nstrange < 10 .or. nstrange >= 500 ) then
                 write(0,*) 'phits: strange ptcl'
                 write(0,*) 'code,sub,charge=', code, subcode,
     *                charge
                 write(0,*)
     *                'i=',i, ' qclust(1~7,i)=', qclust(1:7, i)
                 write(0,*) ' nstrange=', nstrange
                 if( nstrange >= 500) then
                    write(0,*) 'too many '
                    stop
                 endif
              endif
            !  seems to appear for pi- proj.  charge=2/ i is last one
            !  init rn:   238134192   724765212  for epics9.09
            !   write(0,*) ' code ,sub,ch=', code, subcode, charge
            ! lets neglect it
               cycle
            endif
 333        continue
            n = n + 1
            a(n).fm.p(4) = 
     *           sqrt( (qclust(7, i)/1000.d0)**2 + a(n).mass**2) 
                  ! (7,i) is  K.E in MeV
            a(n).fm.p(1) = qclust(1, i) ! in GeV/c
            a(n).fm.p(2) = qclust(2, i)
            a(n).fm.p(3) = qclust(3, i)
         endif
      enddo
c/////////
c      write(0,*) ' exiting cjqmdout'
c//////
      end

==> Cosmos7.581.tar.gz: Cosmos7.581/Import/PHITS/cphitsOut.f <==
      subroutine cphitsOut(n, a)
c       **** note  param00.inc by phits dose not accept implicit none
c            a-h,o-z: double.  i-m; integer
c            New variables here must be explicitly declared
c            without exception 
c      implicit none
#include "Zcode.h"
#include "Zptcl.h"
      include "JQMD/param00.inc"
      integer,intent(out)::n  ! total number of ptcls 
      record /ptcl/ a(*)       !   a(n). each ptcl property
      integer code, subcode, charge

      integer nclst, iclust
*        nclst   : total number of out going particles and nuclei      *
      common /clustf/ nclst, iclust(nnn)
      integer jclust
      real(8):: qclust
      common /clustg/ jclust(0:7,nnn),  qclust(0:12,nnn)

      integer i, j
*                                                                      *
*                j = 0, nucleus                                        *
*                  = 1, proton                                         *
*                  = 2, neutron                                        *
*                  = 3, pion                                           *
*                  = 4, photon                                         *
*                  = 5, kaon                                           *
*                  = 6, muon                                           *
*                  = 7, others                                         *
*                                                                      *

*          jclust(j,:)                                                 *
*                j = 0, angular momentum                               *
*                  = 1, proton number                                  *
*                  = 2, neutron number                                 *
*                  = 3, ip, see below                                  *
*                  = 4,                                                *
*                  = 5, charge                                         *
*                  = 6, baryon number                                  *
*                  = 7, kf code                                        *
*                                                                      *
*        qclust(i,nclst)                                               *
*                                                                      *
*                i = 0, impact parameter                               *
*                  = 1, px (GeV/c)                                     *
*                  = 2, py (GeV/c)                                     *
*                  = 3, pz (GeV/c)                                     *
*                  = 4, etot = sqrt( p**2 + rm**2 ) (GeV)              *
*                  = 5, rest mass (GeV)                                *
*                  = 6, excitation energy (MeV)                        *
*                  = 7, kinetic energy (MeV)                           *
*                  = 8, weight change                                  *
*                  = 9, delay time                                     *
*                  = 10, x-displace                                    *
*                  = 11, y-displace                                    *
*                  = 12, z-displace                                    *
*                                                                      *
*        numpat(i) : total number of out going particles or nuclei     *
*                                                                      *
*                i =  0, nuclei                                        *
*                  =  1, proton                                        *
*                  =  2, neutron                                       *
*                  =  3, pi+                                           *
*                  =  4, pi0                                           *
*                  =  5, pi-                                           *
*                  =  6, mu+                                           *
*                  =  7, mu-                                           *
*                  =  8, K+                                            *
*                  =  9, K0                                            *
*                  = 10, K-                                            *
*                                                                      *
*                  = 11, other particles                               *
*                                                                      *
*                  = 12, electron                                      *
*                  = 13, positron                                      *
*                  = 14, photon                                        *
*                                                                      *
*                  = 15, deuteron                                      *
*                  = 16, triton                                        *
*                  = 17, 3He                                           *
*                  = 18, Alpha                                         *
*                  = 19, residual nucleus                              *
*                                                                      *
*                                                                      *
************************************************************************
      integer kfcode
c/////////
c      write(0,*) 'cphitsOut:  nclst=', nclst
c      write(0,*) '  iclust =',  iclust(1:nclst)
c//////////
      do i=1, nclst
c//////////
c         write(0,*) 'i=', i, ' jclust=',jclust(0:7,i)
c///////////
         if( iclust(i) == 0 ) then
            ! nucleus
            charge = jclust(5,i)
            subcode = jclust(6,i)
            code = kgnuc
         else
            kfcode=jclust(7,i)
c/////////////
c            write(0,*) 'i=',i, "  kfcode =",kfcode, 
c     *           ' iclust=', iclust(i)
c/////////////
            call ckf2cos(kfcode, code, subcode, charge)
         endif
c/////////
c         write(0,*)  ' calling cmkptc'
c////////////
         call cmkptc(code, subcode, charge, a(i))
         
         a(i).fm.p(4) = 
     *    sqrt( (qclust(7, i)/1000.d0)**2 + a(i).mass**2) 
                  ! (7,i) is  K.E in MeV
         a(i).fm.p(1) = qclust(1, i)    ! in GeV/c
         a(i).fm.p(2) = qclust(2, i)
         a(i).fm.p(3) = qclust(3, i)
c/////////////////////
c         write(0,*) ' code ,sub,ch=', code, subcode, charge
c         write(0,*) 'i=',i, ' qclust(1~7,i)=', qclust(1:7, i)
c         write(0,*)
c////////////////   
      enddo
      n = nclst
c/////////
c      write(0,*) ' exiting cjqmdout'
c//////
      end
